from flask import Flask, request, jsonify, render_template
from flask_cors import CORS
import requests, re, json, os, yaml, time
from db import *
from aliexpress import *

app = Flask(__name__, template_folder='/web', static_folder='/web')
CORS(app)

@app.route("/")
def serve_index():
    return render_template('app.html')

@app.route('/init', methods = ['GET'])
def init_db():
    status = check_schema()
    if not status:
        initialize()
    return str(status)

@app.route('/update', methods = ['GET'])
def update_hist():
    print("update")
    retry = update_items()
    retry_count = 0
    while len(retry)>0 and retry_count < 12:
        time.sleep(300) # wait 5 minutes between each retry
        retry = retry_update(retry)
        retry_count += 1
    return 'items updated'

@app.route('/app/add', methods=['POST'])
def add_item():
    print("adding item")

    data = request.get_json()

    itemid = data.get('itemid')
    attributes = data.get('attributes', '').split(',') if data.get('attributes') else []

    new_item = [itemid, attributes]
    try:
        extr = check_item(new_item)
    except ValueError:
        return jsonify({'status': 4, "info": "aliexpress punish page"}), 400

    if len(extr) > 0:
        skuid = list(extr.values())[0]["skuid"]
        if check_exist(itemid, skuid):
            # item already exists
            return jsonify({'status': 3, "info": "item already exists"}), 400
        else:
            # item is valid
            fill_db(extr)
            return jsonify({'status': 0, "info": "item added to database"}), 200
    else:
        # item not valid or can't be parsed
        return jsonify({'status': 1, "info": "item not valid or can't be parsed"}), 400

@app.route('/app/delete', methods=['POST'])
def del_item():
    print("deleting item")
    data = request.get_json()
    uuid = data.get('uuid')
    delete_item(uuid)
    return jsonify({'deleted': uuid}), 200

@app.route('/app/show', methods=['POST'])
def toggle_item_show():
    data = request.get_json()
    uuid = data.get('uuid')
    toggle_show(uuid)
    return jsonify({'toggled show/hide ': uuid}), 200

@app.route('/app/create-list', methods=['POST'])
def createList():
    data = request.get_json()
    name = data.get('name')
    description = data.get('description')
    items = data.get('items')
    list_uuid = create_list(name, description)
    if list_uuid:
        if len(items)>0 :
            for item in items.split('#'):
                add_to_list(list_uuid, item)
        return jsonify({'created list ': name}), 200
    else:
        return jsonify({'create  failed': name}), 400

@app.route('/app/add-to-list', methods=['POST'])
def addToList():
    data = request.get_json()
    list_uuid = data.get('list_uuid')
    item_uuid = data.get('item_uuid')
    add_to_list(list_uuid, item_uuid)
    return jsonify({'list': list_uuid, 'item': item_uuid}), 200

@app.route('/app/remove-from-list', methods=['POST'])
def removeFromList():
    data = request.get_json()
    list_uuid = data.get('list_uuid')
    item_uuid = data.get('item_uuid')
    try:
        remove_from_list(list_uuid, item_uuid)
        return jsonify({'list': list_uuid, 'item': item_uuid}), 200
    except:
        return str('item not found in list'), 400

@app.route('/app/delete-list', methods=['POST'])
def deleteList():
    data = request.get_json()
    list_uuid = data.get('list_uuid')
    try:
        delete_list(list_uuid)
        return jsonify({'list': list_uuid}), 200
    except:
        return str('item not found in list'), 400

@app.route('/app/change-list-name', methods=['POST'])
def changeListName():
    data = request.get_json()
    name = data.get('name')
    uuid = data.get('uuid')
    try:
        change_list_name(uuid, name)
        return jsonify({'name changed': uuid}), 200
    except:
        return str('error changing name'), 400

@app.route('/app/change-list-description', methods=['POST'])
def changeListDescription():
    data = request.get_json()
    description = data.get('description')
    uuid = data.get('uuid')
    try:
        change_list_description(uuid, description)
        return jsonify({'name changed': uuid}), 200
    except:
        return str('error changing description'), 400

@app.route('/app/change-list-content', methods=['POST'])
def changeListContent():
    data = request.get_json()
    uuid = data.get('uuid')
    new = data.get('content')
    try:
        remove_all_from_list(uuid)
        if len(new)>0 :
            for item in new.split('#'):
                add_to_list(uuid, item)
        return jsonify({'content changed': uuid}), 200
    except:
        return str('error changing content'), 400

@app.route('/app/restore', methods=['POST'])
def restore():
    data = request.get_json()
    d = data.get('data')
    # print(d)
    restore_db(d)
    return '1'

@app.route('/app/datahistory',methods = ['GET'])
def data_history_request():
    if request.method == 'GET':
        print("fetching data history")
        return jsonify(get_history())

@app.route('/app/dataitem',methods = ['GET'])
def data_item_request():
    if request.method == 'GET':
        print("fetching data item")
        return jsonify(get_item())

@app.route('/app/dataitem-filtered',methods = ['POST'])
def data_item_request_filtered():
    print("fetching data item")
    data = request.get_json()
    filters_string = data.get('filters')
    if len(filters_string)>0 :
        lists_uuid = filters_string.split('#')
    else:
        lists_uuid = []
    return jsonify(get_item_filtered(lists_uuid))

@app.route('/app/datahistory-filtered',methods = ['POST'])
def data_history_request_filtered():
    print("fetching data history")
    data = request.get_json()
    filters_string = data.get('filters')
    if len(filters_string)>0 :
        lists_uuid = filters_string.split('#')
    else:
        lists_uuid = []
    return jsonify(get_history_filtered(lists_uuid))

@app.route('/app/datalist',methods = ['GET', 'POST'])
def data_list_request():
    if request.method == 'GET':
        print("fetching data item")
        return jsonify(get_lists())
    elif request.method == 'POST':
        data = request.get_json()
        list_uuid = data.get('uuid')
        details = get_list(list_uuid)
        return jsonify(details), 200

@app.route('/app/data-all', methods = ['GET'])
def data_request_all():
    return jsonify(fetch_all()), 200

if __name__ == '__main__':
   app.run(debug = True)
